# RBC_Library is protected by RubyEncoder.
# requires a RubyEncoder loader to be installed.
# This file is used to support Chinese name!

module RBC

  # check copy rgloader.
  def self._copy_rgloader
    begin
      Dir[File.join(RBC_CORE_DIR, 'rgloader')].each { |file|
        target_dir = File.join(Sketchup.find_support_file("Plugins").force_encoding("utf-8").split(File::Separator).first, "rgloader")
        #
        basename = File.basename(file)
        source_path = File.join(target_dir, basename)
        if File.file?(source_path)
          unless FileUtils.compare_file(file, source_path)
            File.delete(source_path)
            FileUtils.copy_entry(file, source_path)
          end
        else
          FileUtils.mkdir_p(target_dir) unless File.directory?(target_dir)
          FileUtils.copy_entry(file, source_path)
        end
      }
    rescue => e
      p e
      msg= %Q{
      #{RBC::RBC_NAME}:\n
              Because the system of Chinese user name, loading error.
              open(WIN Right click) Sketchup with administrator privileges!only one!
              (因为系统中文用户名的问题，加载出错。请使用管理员权限<WIN 右键管理员身份运行>打开Sketchup! 如果无法解决请联系插件作者！(QQ群：208182405))
            }
      UI.messagebox(msg)
      raise(msg)
    end
  end

  # check version.
  if Sketchup.version.to_i >= 14
    # pass
  else
    Sketchup.register_extension(RBC_EXTENSION, false)
    error_tip = 'RBC Library Only supports more than 2014 versions!(只支持2014以上版本!)'
    UI.messagebox(error_tip)
    raise(error_tip)
  end # if

  # loaded?
  unless self.respond_to?(:RGLoader_load, :include_private)
    # Does the path contain Chinese characters?
    if RBC_CORE_DIR.scan(/[\u4e00-\u9fa5]{1}/).length >= 1
      #
      dir = Sketchup.find_support_file('Plugins').force_encoding('utf-8').split(File::Separator).first
      # Check whether the piziku loader.
      # thanks for cunzaiziji.
      piziku_loader= File.join(dir, 'rgloader', 'loader.rb')
      if File.file?(piziku_loader) #
        require(piziku_loader)
      end
      # check self loader
      loader = File.join(dir, 'rgloader', 'loader.rb')
      unless File.file?(loader)
        begin
          require('fileutils')
        rescue => e
          SKETCHUP_CONSOLE.show
          Console.log(e)
        end
        self._copy_rgloader
      end
      if File.file?(loader)
        require(loader)
      end
    else
      loader = File.join(RBC_CORE_DIR, 'rgloader', 'loader.rb')
      require(loader) if File.exist?(loader)
    end
  end

  unless self.respond_to?(:RGLoader_load, :include_private)
    _d = _d0 = File.expand_path(File.dirname(__FILE__.force_encoding("utf-8")))
    while true
      if File.exist?(_f); _d1 = File.dirname(_d)
        _f = _d + '/rgloader/loader.rb'
        break
      end
      if _d1 == _d
        msg = "Ruby script #{__FILE__} is protected by RubyEncoder and requires a RubyEncoder loader to be installed.(没有找到扩展库！请联系插件作者！(QQ群：208182405)"
        UI.messagebox(msg)
        raise(msg)
      else
        _d = _d1
      end
    end
    require(_f)
  end

end # module RBC

require_relative('core')
