/*
 * ----------------------------------
 * RBC.GUI.HtmlDialog
 *
 * Sugar
 * Date   : 2017.9.28
 *
 * ----------------------------------
 * */

/*
 * **************************
 * ERROR
 * **************************
 * */

/*
 * Capture global JS errors and output to the console of SketchUp.
 * */
window.onerror = function (message, source, lineno, colno, error) {
    sketchup._error({
        "message": message.toString(),
        "source": source.toString(),
        "lineno": lineno.toString(),
        "colno": colno.toString()
    });
};

/*
 * **************************
 * log_info
 * **************************
 * */

/*
 * Output information to the browser console.
 * */
function log_info(message) {
    if (window.console != undefined) {
        console.log(message);
    }
}

var RBC = RBC || {};

/*
 * **************************
 * Language
 * **************************
 * */

//
RBC.Language = {
    /*
     * language JSON
     * */
    strings: {},

    /*
     * Initialization language value
     * */
    refresh: function () {
        log_info("refresh RBC.Language.");
        if (/SketchUp/i.test(navigator.userAgent)) {
            if (arguments.length == 1 && typeof(arguments[0]) === "function") {
                this._refreshFun = arguments[0];
            }
            RBC.GUI.HtmlDialog.callback({
                'name': '_language',
                'data': true,
                'success': function (data) {
                    RBC.Language.strings = data
                    RBC.Language._refreshFun(RBC.Language)
                }
            })
            // sketchup._language(true,{
            //     onCompleted: function() {
            //         RBC.Language._refreshFun(RBC.Language)
            //     }
            // });
        } else {
            if (arguments.length == 1 && typeof(arguments[0]) === "function") {
                this._refreshFun = arguments[0];
            }
            this._refreshFun(RBC.Language)
        }
        return null;
    },

    /*
     * Get language value
     * */
    tr: function (key) {
        if (!/SketchUp/i.test(navigator.userAgent)) {
            log_info("RBC.Language: " + key + "<==>" + key);
            return key;
        }
        if (this.strings[key]) {
            log_info("RBC.Language: " + key + "<==>" + this.strings[key]);
            return this.strings[key];
        } else {
            RBC.GUI.HtmlDialog.callback('_language_insert_key', key);
            //sketchup._language_insert_key(key);
            return key;
        }
    },
    /*
     * Store the refresh callback function.
     * */
    _refreshFun: function () {
    }

};

/*
 * **************************
 * HtmlDialog
 * **************************
 * */

RBC.GUI = {};
RBC.GUI.HtmlDialog = {

    /* close dialog.
     * @method: close
     * */
    close: function () {
        this.callback('_command', 'close_dlg');
    },

    /* Wrapper to call any defined action_callback, handles escaping and encoding etc.
     * @param json
     * {
     *    name: string,
     *    data: json,
     *    test: function,
     *    success: function,
     *    error: function,
     *    complete: function
     * }
     * @param name, *arguments
     */
    callback: function () {
        // merge the given arguments with
        var opt = {
                success: function () {
                }, // null,
                error: function () {
                }, // null,
                complete: function () {
                }  // null
            },
            name = null,
            args = [];
        if (arguments.length == 1 && typeof(arguments[0]) === 'object') {
            json = arguments[0];
            for (var key in json) {
                opt[key] = json[key]
            }
            name = opt.name;
            args = opt.data;
            //
            if (!/SketchUp/i.test(navigator.userAgent)) {
                if (typeof opt.test !== 'undefined') {
                    value = opt.test(args);
                    opt.complete(value);
                    return true;
                } else {
                    log_info('Error(J): Please add the Test function key!');
                    return false;
                }
            }
        } else if (arguments.length >= 1) {
            name = arguments[0];
            args = Array.prototype.slice.call(arguments).slice(1);
            //
            if (!/SketchUp/i.test(navigator.userAgent)) {
                log_info('Error(J): Please execute it in SU!');
                return false
            }
        } else if (arguments.length == 1 && typeof(arguments[0]) === "string") {
            name = arguments[0];
            args = null;
        } else {
            log_info('Error(J): Incorrect parameter!');
            return false
        }
        var id = (this._messageIdCounter++);
        this._callbacks[id] = opt;
        this._messages.push({
            'fromFrame': 'HtmlDialog',
            'name': name,
            'id': id,
            'data': args
        });
        if (this._ready) {
            this._nextMessage()
        }
    },

    /*
     *
     *
     * */
    _nextMessage: function () {
        var message = this._messages.shift();
        if (!message) {
            this._ready = true;
            return;
        }
        this._ready = false;
        var js_command = 'sketchup.' + message.name + '(' + JSON.stringify(message) + ')';
        try {
            eval(js_command);
        } catch (e) {
            throw new Error('ERROR(J): An exception occurred with the JS method:(sketchup.action);ActionName:(' + message[0] + ')' + e.toString());
        }
    },

    _callTo: function (params) {
        var returnValue = null;
        try {
            //Create the function
            var fn = eval(params.funName); // alternative only for global functions: window[functionName]
            if (fn == null) {
                throw "function is not defined"
            }
            var data = eval(params.arguments);
            returnValue = fn.apply(fn, data);
            sketchup._callTo({
                'id': params.id,
                'type': 'success',
                'value': returnValue
            });
        } catch (e) {
            sketchup._callTo({
                'id': params.id,
                'type': 'error',
                'value': e.toString()
            });
        }
    },

    /*
     *
     * */
    _cleanUp: function (id) {
        delete this._callbacks[id];
    },
    _ready: true,
    _messages: [],
    _id_label: 'RBCFrameId_',
    _messageIdCounter: 0,
    _callbacks: {}
};

