# 基础几何模块

![image](geom.jpg)

## 基础几何概念
- FiniteLine(Fline)  => 有起点和终点的一条直线
- Ray                => 有起点没有终点的一条射线，没有终点，无限延伸
- Line               => 没有起点和终点、两边无限延伸
- Vector             => 向量 没有位置、有大小有方向
- Point              => 空间点，只有位置，没有大小
- Plane              => 无限平面 没有厚度，无限延伸
- BB                 => 轴向包围盒
- OBB                => 方向包围盒
- BCube              => 包围圆柱
- BSphere            => 包围球
- Polygon            => 多边形
- ConvexPolygon(CPolygon) =>  凸多边形
- Triangle           => 三角形
- Polyline           => 多段线(SU中曲线实际上是多段线)
- Curve              => 曲线

*正式的代表可以直接在API中使用,而非正式 代表不是sketchup官方提倡使用，但是这样做并不违背逻辑，可以灵活使用。*

### 1. 三个数字（Numeric）组成的数组（Array）可以代表空间点（Geom::Point3d）或者向量（Geom::Vector3d）

**[ Numeric, Numeric, Numeric ]**

这是官方提供的方式之一。
绝大部分情况下这没有什么问题。
空间点（Geom::Point3d）或者向量（Geom::Vector3d）都有自己的类。
而且不要以为数组表达方法和一个对象表达方式相同，并不完全相同。
尤其是reverse方法，在数组中和（Vector3d）中完全不是一个意思。

### 2.  两个空间点（Geom::Point3d）组成的数组（Array）

**[ Geom::Point3d, Geom::Point3d ]**

* Fline （非正式）
* Ray
* Line
* Vector （非正式，第一个点指向第二个点）

### 3.  一个空间点（Geom::Point3d）和一个向量（Geom::Vector3d）组成的数组（Array）。

注意顺序，空间点（Point3d）在前，向量（Vector3d）在后。

**[ Geom::Point3d, Geom::Point3d ]**

* Line
* Ray
* Plane

### 4.Line、Ray和Plane的表达方式都可以是 [Geom::Point3d, Geom::Point3d]

那么我们怎么去区分呢？
为什么要区分? 你把它当作Ray，它便是Ray，你把它当作Line，它便是Line。

### 5.四个数字（Numeric）组成的数组（Array）

**[ Numeric, Numeric, Numeric, Numeric ]**

Plane 前三个数代表这个plane的法向量，最后一个则是原点ORIGIN到PLANE的最短距离。

### 6.空间点（Point3d）的数组（Array）可以代表曲线（Curve）

**[ Geom::Point3d, Geom::Point3d，... ]**

数组（Array）的顺序就是曲线的顺序。
当然，按顺序存放的直线（Edge）的数组（Array）也可以代表曲线（Curve），这是非正式的选择。

-------------------------------

这里所说的并不是唯一法则，而是根据抽象思维提炼的，法则并不重要，重要的是抽象。
你完全可以抽象出一套自己的规则，只要它符合逻辑。
如果是一种描述方式就是一种抽象，那么，几何的表达式方法也是一种抽象。
抽象只是用来描述或者计算的手段，不要当成目的。目的是解决问题，不是吗？
抽象无处不在，不可逃避。
因为抽象是高级学习能力的一部分。
抽象不是复杂化，恰恰是复杂问题的简单描述。

-------------------------------

## 基础几何概念的表达式描述

### 1.Line

### 2.Ray

### 3.Plane

### 4.Fline

### 5.Point

### 6.Vector

# face方向和点顺序之间的关系。


-------------------------------

# 地理坐标系（Geographic Coordinate System）

### UTM

### 

-------------------------------

# 包围盒和碰撞检测
包围盒算法是一种求解离散点集最优包围空间的方法。
基本思想是用体积稍大且特性简单的几何体（称为包围盒）来近似地代替复杂的几何对象。

碰撞检测问题在虚拟现实、计算机辅助设计与制造、游戏及机器人等领域有着广泛的应用，甚至成为关键技术。
而包围盒算法是进行碰撞干涉初步检测的重要方法之一。

碰撞检测技术中所用的包围盒有两个属性：简单性和紧密性。
简单性是指包围盒间进行相交测试时需要的计算量，这不但要求几何形状简单容易计算，而且要求相交测试算法简单快速；紧密性要求包围盒尽可能的贴近被包围的对象，这一属性直接关系到需要进行相交测试的包围盒的数目，紧密性越好，参与相交测试的包围盒数目就越少。


## 最常见的包围盒算法有
* AABB包围盒（Axis-aligned bounding box），
包围球（Sphere），
方向包围盒OBB（Oriented bounding box）
以及固定方向凸包FDH（Fixed directions hulls或k-DOP）。

AABB
AABB是应用最早的包围盒。它被定义为包含该对象，且边平行于坐标轴的最小六面体。
故描述一个AABB，仅需六个标量。
AABB构造比较简单，存储空间小，但紧密性差，尤其对不规则几何形体，冗余空间很大，
当对象旋转时，无法对其进行相应的旋转。
处理对象是刚性并且是凸的，不适合包含软体变形的复杂的虚拟环境情况。

* Sphere
对象的包围球被定义为包含该对象的最小的球体。
确定包围球，首先需分别计算组成对象的基本几何元素集合中所有元素的顶点的x，y，z坐标的均值以确定包围球的球心，
再由球心与三个最大值坐标所确定的点间的距离确定半径r。
包围球的碰撞检测主要是比较两球间半径和与球心距离的大小。

* OBB是较为常用的包围盒类型。
它是包含该对象且相对于坐标轴方向任意的最小的长方体。
OBB最大特点是它的方向的任意性，这使得它可以根据被包围对象的形状特点尽可能紧密的包围对象，
但同时也使得它的相交测试变得复杂。
OBB包围盒比AABB包围盒和包围球更加紧密地逼近物体，能比较显著地减少包围体的个数，
从而避免了大量包围体之间的相交检测。
但OBB之间的相交检测比AABB或包围球体之间的相交检测更费时。

* FDH（k-DOP）是一种特殊的凸包，继承了AABB简单性的特点，
但其要具备良好的空间紧密度，必须使用足够多的固定方向。
被定义为包含该对象且它的所有面的法向量都取自一个固定的方向(k个向量)集合的凸包。
FDH比其他包围体更紧密地包围原物体，创建的层次树也就有更少的节点，求交检测时就会减少更多的冗余计算，但相互间的求交运算较为复杂。

## 优缺点
AABB也是比较简单的一类包围盒。
但对于沿斜对角方向放置的瘦长形对象，其紧密性较差。
由于AABB相交测试的简单性及较好的紧密性，因此得到了广泛的应用，还可以用于软体对象的碰撞检测。

包围球是比较简单的包围盒，而且当对象发生旋转运动时，包围球不需做任何更新，
当几何对象进行频繁的旋转运动时，使用包围球可能会得到很好的结果；
当对象变形时，需要重新计算其包围球。但它的紧密性是比较差的，因此较少使用。

OBB的简单性要比上面两种包围盒差，但它的紧密性是比较好的，
可以大大减少参与相交测试的包围盒的数目，因此总体性能要优于AABB和包围球。
当几何对象发生旋转运动后，只要对OBB进行同样的旋转即可。
因此，对于刚体间的碰撞检测，OBB不失为一种较好的选择，
但迄今为止，还没有一种有效的方法能够较好地解决对象变形后OBB树的更新问题，
而重新计算每个结点的OBB的代价又太大，所以OBB不适用于包含软体对象的复杂环境中。
FDH相对于上面三种包围盒其紧密性是最好的，同时其相交测试算法比OBB要简单的多。
可以用于软体对象的碰撞检测。
## 应用
在计算机图形学与计算几何领域，一组物体的包围盒就是将物体组合完全包容起来的一个封闭空间。
将复杂物体封装在简单的包围盒中，用简单的包围盒形状来近似代替复杂几何体的形状，
就可以提高几何运算的效率。并且通常简单的物体比较容易检查相互之间的重叠。

## 碰撞检测
可以根据情况分为静态碰撞、伪动态碰撞、动态碰撞。
空间分解法和层次包围体

空间分解法有八叉树法和二叉树空间剖分法。
包围盒应该尽可能的逼近物体。相交测试应该尽可能快

